function out=gLogMagyReadSeaSpy(fName,Delim,EqId)
%Read SeaSpy data (Standard Format) from gLog
%function out=gLogMagyReadSeaSpy(fName,LDelim,AltCh), where
%fName- path to folder/file with SeaSpy Magnetometer data;
%Delim- [left_delimiter_symbol right_delimiter_symbol] for gLog record;
%EqId- serial numbers [mag, alt, depth], NaN is "no devise";
%out- output structure, which includes fields: CompDay, CompTime, MagDay, MagTime, MagAbsT, MagPrecSignal, MeasurTime, Depth, Altitude, Leak, SignalQuality, FlagsWPMG.
%Example: out=gLogMagyReadSeaSpyGrad('c:\temp\whghghhe\TEST.Nav.txt','~',1);
%--------------------------------------------------------------------------------------
%Log-file (created by gComLog program) example:
%~42042805,*05.001/10:27:11.0 F:050945.927 S:178 D:+010.3m A:001.02m L0 0349ms Q:43    G
%where
%~ - left delimiter (symbol 1);
%42042805 - second per day./1000 (symbols 2-9);
%, - right delimiter (symbol 10);
%*19.212/... - SeaSpy magnetometer data.
%--------------------------------------------------------------------------------------
%*05.001/10:27:11.0 F:050945.927 S:178 D:+010.3m A:001.02m L0 0349ms Q:43    G
%*05.001/10:27:12.0 F:050948.573 S:177 D:+010.3m A:001.02m L0 0372ms Q:54    G
%*YY.JJJ/HH:MM:SS.S F:FFFFFF.FFF S:SSS D:+DDD.Dm A:AAA.A L:L TTTms Q:QQ !!!! CR LF
%Y- Year; JJJ- Julian day; HH- Hour; MM- Minute; SS.S- Second;
%FFFFFF.FFF- Magnetic field (nT);
%SSS- Signal strength of reading. This is a raw number generated by the magnetometer that gives (in part) a good indication of the quality of the final total field measurement. Anything over 80 is considered an acceptable signal, and anything over 130 is considered excellent (SSS should be between 130 and 200 for good quality readings).
%DDD.D- Towfish Depth. The value shown is in meters. The depth sensor can be calibrated using the P and p commands.
%AAA.A- Towfish Altitude. The value shown is in meters. If no altimeter is installed, this field will not be present. If an altimeter is installed, but it cannot obtain a lock on the seafloor (for example if it is too far away) this value will be 000.0m.
%L- Leak sensor output, 0-9. 0 indicates no leak, and 9 indicates that a leak is present.
%TTT- Measurement time. Ideally, this should be the magnetometers cycling time minus 35ms, with a maximum of 965ms (965 when F is greater than 42000 and 465 when F is less than 42000). If you see a G message, indicating that measurement was prematurely terminated due to a high gradient condition, this value will tell you how severe the gradient is.
%QQ- Signal quality. This is a two-digit number between 00 to 99. The left digit is a good indication of signal strength, and the right digit indicates how much information was available for measurement.
%!!!!- Warning Messages "WPMG" or "    ": 
%W- Weak signal. This message is displayed if the signal strength for the reading is below a threshold value;
%P- Poor reading. This message is displayed if the signal is sampled for too short a time period, for whatever the reason. Expect this message under conditions of extremely high magnetic gradient.
%M- Instrument Mistuned. The magnetometer may decide to display this message under extremely poor signal conditions, which is characteristic of poor tuning settings. When this message occurs, the instrument will attempt to retune by executing an initialize tuning procedure, if the auto-tuning feature is enabled.
%G- Gradient condition. In high magnetic gradients, the signal produced by the sensor decays more quickly. This message occurs if the measurement time was prematurely terminated due to a quickly decaying signal. The strength of the gradient can be estimated by observing the measurement time. Note that sensitivity will decrease as the measurement time decreases.
%CR LF- Carriage Return (ASCII code 13), Line Feed (ASCII code 10).
%--------------------------------------------------------------------------------------

if (size(fName,1)==1)&&(fName(end)=='\'), dz=dir(fName);dz([dz.isdir])=[];fName=[repmat(fName,length(dz),1) char(dz.name)];end; %fName=sortrows(fName);
out=struct('CompDay',[],'CompTime',[],'MagDay',[],'MagTime',[],'MagAbsT',[],'MagPrecSignal',[],'MeasurTime',[],'Depth',[],'Altitude',[],'Leak',[],'SignalQuality',[],'FlagsMGWP',[]);
for nn=1:size(fName,1),
    fNameN=deblank(fName(nn,:));
    %disp(fNameN);
    [fId, mes]=fopen(fNameN,'r');if ~isempty(mes), error(mes);end;
    if isnan(EqId(2)),C=textscan(fId,'%c%f%c*%f/%f:%f:%f F:%f S:%f D:%fm L%f %fms Q:%f %c%c%c%c\r\n','Delimiter',' ','MultipleDelimsAsOne',0); C=[C{1:10} {zeros(size(C{1}))} C{11:17}];end;%no altimeter
    if ~isnan(EqId(2)),C=textscan(fId,'%c%f%c*%f/%f:%f:%f F:%f S:%f D:%fm A:%fm L%f %fms Q:%f %4c\r\n','Delimiter',' ','MultipleDelimsAsOne',0);end;
    fclose(fId);
    if any(C{1}~=Delim(1)), error('LeftDelimiter not true');end;
    if any(C{3}~=Delim(2)), error('RightDelimiter not true');end;
    CompTime=C{2}'./1000;
    L=find(fNameN=='\');fNameDay=fNameN(L(end)+1:L(end)+8);tmp=datenum(str2double(fNameDay(1:4)),str2double(fNameDay(5:6)),str2double(fNameDay(7:8)));CompDay=repmat(tmp,size(CompTime));
    MagDay=datenum(2000+fix(C{4}),0,(C{4}-fix(C{4}))*1000)';
    MagTime=(C{5}*3600+C{6}*60+C{7})';
    %    FlagsGWP=uint8([(C{13}=='G').*4+(C{14}=='W').*2+(C{15}=='P').*1 (C{21}=='G').*4+(C{22}=='W').*2+(C{23}=='P').*1])';

    FlagsMGWP=uint8((C{15}(:,1)=='W').*2+(C{15}(:,2)=='P').*1+(C{15}(:,3)=='M').*8+(C{15}(:,4)=='G').*4)';
    %add structs
    out1=struct('CompDay',CompDay,'CompTime',CompTime,'MagDay',MagDay,'MagTime',MagTime,'MagAbsT',C{8}','MagPrecSignal',C{9}','MeasurTime',C{13}','Depth',C{10}','Altitude',C{11}','Leak',C{12}','SignalQuality',C{14}','FlagsMGWP',FlagsMGWP);
    out=gFieldsRowAppend(out,out1,size(out.CompDay,2));
end;

%mail@ge0mlib.com 30/09/2019